/*
 * GNOME QuickRes Applet - Change the X11 display mode from the GNOME taskbar
 * Copyright (C) 1999  Vicente Aguilar <vaguilar@retemail.es>
 * http://pagina.de/bisente/gquickres
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* Ok, the code is a mess, I know ... :)      */
/* But, what did you spected from a           */
/* Quick-And-Dirty-Hack(tm) of other applets? */
/* It works, that's enough for me. ;)         */


#include <gnome.h>
#include <applet-widget.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <X11/Xlib.h>
#include <X11/extensions/xf86dga.h>
#include <X11/extensions/xf86vmode.h>
#include <stdio.h>
#include "small_icon.xpm"
#include "plus_sign.xpm"
#include "minus_sign.xpm"

#ifdef ENABLE_NLS
#    include <libintl.h>
#    define _(String) gettext (String)
#    ifdef gettext_noop
#        define N_(String) gettext_noop (String)
#    else
#        define N_(String) (String)
#    endif
#else
/* Stubs that do something close enough.  */
#    define textdomain(String) (String)
#    define gettext(String) (String)
#    define dgettext(Domain,Message) (Message)
#    define dcgettext(Domain,Message,Type) (Message)
#    define bindtextdomain(Domain,Directory) (Domain)
#    define _(String) (String)
#    define N_(String) (String)
#endif
/* </gettext_stuff> */


static gint button_press (GtkWidget *);
void on_destroy (void);

GtkWidget *applet;
GtkWidget *quickres;

Display *dpy;
XF86VidModeModeInfo **vm_modelines; 
int vm_count;

typedef enum { PREVIOUS_MODE=-1, NEXT_MODE=+1 } switch_mode_action;


static void
applet_set_default_back()
{
	g_return_if_fail (quickres != NULL);

	gtk_widget_set_rc_style(quickres);
	gtk_widget_queue_draw(GTK_WIDGET(quickres));
}

static void
applet_set_back_color(GdkColor *color)
{
	GtkStyle *ns;

	ns = gtk_style_copy(quickres->style);
	gtk_style_ref(ns);

	ns->bg[GTK_STATE_NORMAL] = *color;
	ns->bg[GTK_STATE_NORMAL].pixel = 1; /* bogus */

	if(ns->bg_pixmap[GTK_STATE_NORMAL]) {
		gdk_imlib_free_pixmap(ns->bg_pixmap[GTK_STATE_NORMAL]);
		ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;
	}

	gtk_widget_set_style(quickres, ns);

	gtk_style_unref(ns);

	gtk_widget_queue_draw(GTK_WIDGET(quickres));

}


static void
applet_set_back_pixmap(gchar *pixmap)
{
	GdkImlibImage *im;
	GdkPixmap *p;
	GtkStyle *ns;

	if(!pixmap || strcmp(pixmap,"")==0) {
		ns = gtk_style_copy(quickres->style);
		gtk_style_ref(ns);

		p = ns->bg_pixmap[GTK_STATE_NORMAL];
		if(p)
			gdk_imlib_free_pixmap (p);
		ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;

		gtk_widget_set_style(quickres, ns);

		gtk_style_unref(ns);

		return;
	}

	if (!g_file_exists (pixmap))
		return;

	im = gdk_imlib_load_image (pixmap);
	if (!im)
		return;

	gdk_imlib_render (im, im->rgb_width, im->rgb_height);

	p = gdk_imlib_move_image (im);

	ns = gtk_style_copy(quickres->style);
	gtk_style_ref(ns);

	if(ns->bg_pixmap[GTK_STATE_NORMAL])
		gdk_imlib_free_pixmap (ns->bg_pixmap[GTK_STATE_NORMAL]);
	ns->bg_pixmap[GTK_STATE_NORMAL] = p;

	gtk_widget_set_style(quickres, ns);

	gtk_style_unref(ns);


	gdk_imlib_destroy_image (im);
}


static void
applet_back_change(GtkWidget *w,
		   PanelBackType type,
		   gchar *pixmap,
		   GdkColor *color,
		   gpointer data)
{
	if(type == PANEL_BACK_PIXMAP)
		applet_set_back_pixmap(pixmap);
	else if(type == PANEL_BACK_COLOR)
		applet_set_back_color(color);
	else
		applet_set_default_back();
}


static void change_mode(int mode)
{

	int new_mode = mode;
	XF86VidModeSwitchToMode(dpy,XDefaultScreen(dpy),vm_modelines[new_mode]);
   	XFlush(dpy);

}

static void about_cb(AppletWidget *widget, gpointer data)
{ 

	GtkWidget *about;
	static const gchar *author[]= {
		"Vicente Aguilar <vaguilar@retemail.es>" , NULL };

	about = gnome_about_new (
		_("GNOME QuickRes Applet"), 
		VERSION,
		"Copyright (C) Vicente Aguilar 1999",
		author,
		_("Change the X11 display mode from the GNOME taskbar."),
		NULL
	);

	gtk_widget_show(about);

}


static gint button_press (GtkWidget *widget)
{
	  
	int i;
	char mode_name[32];
	
	GtkWidget *menu_items;
	GtkWidget *menu;
	
	/* BEGIN WITH THE GTK WIDGETS */
	menu = gtk_menu_new();
	
	/* PRINT THE LIST OF MODES FOUND */
	for(i=0; i < vm_count; i++) {

		sprintf(mode_name,"%dx%d",vm_modelines[i]->hdisplay,vm_modelines[i]->vdisplay);

		
		menu_items = gtk_menu_item_new_with_label(mode_name);
		
		gtk_menu_append(GTK_MENU(menu), menu_items);
		gtk_signal_connect_object(GTK_OBJECT(menu_items),"activate",
						GTK_SIGNAL_FUNC(change_mode),
						(gpointer) i);
		
		gtk_widget_show(menu_items);
								
	}
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, 0);
	return 0;

}


static void switch_mode_cb( int action )
{

	int new_mode;
	int num_modes;
	XF86VidModeModeInfo **modelines; 

	dpy=XOpenDisplay("");   

	/* GET LIST OF MODES FOUND */
	XF86VidModeGetAllModeLines(dpy,XDefaultScreen(dpy),&num_modes,&modelines);

	if( action == NEXT_MODE )
		new_mode = 1;
	else
		new_mode = num_modes-1;

	XF86VidModeSwitchToMode(dpy,XDefaultScreen(dpy),modelines[new_mode]);
   	XFlush(dpy);
	
	XFree(modelines);

}


void on_destroy(void) {

	XFree(vm_modelines);
	exit(0);

}



GtkWidget *quickres_widget()
{ 

    GtkWidget *fixed;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *button_screen;
	GtkWidget *button_next;
	GtkWidget *button_previous;
	GtkWidget *pixmap_screen;
	GtkWidget *pixmap_next;
	GtkWidget *pixmap_previous;


    fixed = gtk_fixed_new();


    button_screen = gtk_button_new();
	gtk_widget_set_usize(button_screen, 24, 24);
    GTK_WIDGET_UNSET_FLAGS(button_screen, GTK_CAN_DEFAULT);
    GTK_WIDGET_UNSET_FLAGS(button_screen, GTK_CAN_FOCUS);
	gtk_button_set_relief(GTK_BUTTON(button_screen), GTK_RELIEF_NONE);
    pixmap_screen = gnome_pixmap_new_from_xpm_d(small_icon_xpm);
	gtk_container_add(GTK_CONTAINER(button_screen), pixmap_screen);
    
    gtk_signal_connect(GTK_OBJECT(button_screen), "button_press_event",
                       GTK_SIGNAL_FUNC(button_press), NULL);


	vbox = gtk_vbox_new(FALSE, FALSE);
	hbox = gtk_hbox_new(FALSE, FALSE);


	button_next = gtk_button_new();
    GTK_WIDGET_UNSET_FLAGS(button_next, GTK_CAN_DEFAULT);
    GTK_WIDGET_UNSET_FLAGS(button_next, GTK_CAN_FOCUS);
	gtk_button_set_relief(GTK_BUTTON(button_next), GTK_RELIEF_NONE);
    pixmap_next = gnome_pixmap_new_from_xpm_d(plus_sign_xpm);
    gtk_container_add(GTK_CONTAINER(button_next), pixmap_next);
	gtk_box_pack_start(GTK_BOX(hbox), button_next, FALSE, TRUE, 1);

    gtk_signal_connect_object( GTK_OBJECT(button_next), 
                               "button_press_event",
                               GTK_SIGNAL_FUNC(switch_mode_cb), 
                               (gpointer)NEXT_MODE
                             );


	button_previous = gtk_button_new();
    GTK_WIDGET_UNSET_FLAGS(button_previous, GTK_CAN_DEFAULT);
    GTK_WIDGET_UNSET_FLAGS(button_previous, GTK_CAN_FOCUS);
	gtk_button_set_relief(GTK_BUTTON(button_previous), GTK_RELIEF_NONE);
    pixmap_previous = gnome_pixmap_new_from_xpm_d(minus_sign_xpm);
    gtk_container_add(GTK_CONTAINER(button_previous), pixmap_previous);
	gtk_box_pack_start(GTK_BOX(hbox), button_previous, FALSE, TRUE, 2);

    gtk_signal_connect_object( GTK_OBJECT(button_previous), 
                               "button_press_event",
                               GTK_SIGNAL_FUNC(switch_mode_cb), 
                               (gpointer)PREVIOUS_MODE
                             );


	gtk_box_pack_start(GTK_BOX(vbox), button_screen, FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 2);
	gtk_fixed_put (GTK_FIXED(fixed), vbox, 0, 0);


	applet_widget_register_stock_callback(
		APPLET_WIDGET(applet),
		"about",
		GNOME_STOCK_MENU_ABOUT,
		_("About..."),
		about_cb,
		NULL
	);


	gtk_widget_show_all(fixed);
	return (fixed);

}


int main (int argc, char **argv)
{

	/* initialize i18n stuph */
	bindtextdomain(PACKAGE, LOCALE);
	textdomain(PACKAGE);
	
	applet_widget_init("quickres_applet",VERSION,argc,argv,NULL,0,NULL);
	
	applet = applet_widget_new("quickres_applet");
	if(!applet) {
		g_print(_("Cannot create applet!\n"));
		_exit(127);
	}
	
	dpy=XOpenDisplay("");   

	/* GET LIST OF MODES FOUND */
	XF86VidModeGetAllModeLines(dpy,XDefaultScreen(dpy),&vm_count,&vm_modelines);
	
	if(!(vm_count)) {
		g_print(_("gtkmodes error: could not find any modes!\n"));
		exit(1);
	}

	gtk_widget_realize(applet);

	quickres = quickres_widget();
	gtk_widget_show(quickres);

	gtk_signal_connect(GTK_OBJECT(applet),"back_change",
			   GTK_SIGNAL_FUNC(applet_back_change),
			   quickres);

	applet_widget_add(APPLET_WIDGET(applet),quickres);
	gtk_widget_show(applet);

	applet_widget_set_tooltip( APPLET_WIDGET(applet),
		_("Click on the screen icon to select a mode from the list, or use "
		  "the '+' and '-' icons to switch to the next or previous mode.") );

	applet_widget_gtk_main();
	
	return 0;

}       
